package com.rf.AIquantum.dialogue.rest;

import cn.hutool.core.date.DateUtil;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.auth0.jwt.interfaces.DecodedJWT;
import com.rf.AIquantum.base.rest.BaseController;
import com.rf.AIquantum.dialogue.dao.model.ChatHistoryEntity;
import com.rf.AIquantum.dialogue.dao.model.DialogueEntity;
import com.rf.AIquantum.dialogue.service.ChatHistoryService;
import com.rf.AIquantum.dialogue.service.DialogueService;
import com.rf.AIquantum.user.dao.model.UserEntity;
import com.rf.AIquantum.user.service.UserService;
import com.rf.AIquantum.utils.Constant;
import com.rf.AIquantum.utils.JWTUtil;
import com.rf.AIquantum.utils.Result;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.lang.StringUtils;
import org.apache.http.HttpEntity;
import org.apache.http.HttpResponse;
import org.apache.http.client.methods.HttpPost;
import org.apache.http.entity.StringEntity;
import org.apache.http.entity.mime.MultipartEntityBuilder;
import org.apache.http.impl.client.CloseableHttpClient;
import org.apache.http.impl.client.HttpClients;
import org.apache.http.util.EntityUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

/**
 * @Description: 对话相关接口
 * @Author: zsy
 * @Date: 2024/12/4
 */
@RestController
@RequestMapping("/dialogue")
@Api(tags = "对话相关接口")
public class DialogueController extends BaseController {

    @Autowired
    private UserService userService;

    @Autowired
    private DialogueService dialogueService;

    @Autowired
    private ChatHistoryService chatHistoryService;

    @PostMapping("/saveChat")
    @ApiOperation(value = "保存对话",notes = "参数包括：phone:手机号, dialogueId:对话id（为空时是新建对话）, content:消息内容,image：图片（Base64格式，可以为空）")
    public Result saveChat(@RequestBody String jsonParam) throws UnsupportedEncodingException {
        JSONObject jsonObject =JSONObject.parseObject(jsonParam);
        if (!jsonObject.containsKey("phone")) {
            return fail("", "缺少参数：phone");
        }
        if (!jsonObject.containsKey("dialogueId")) {
            return fail("", "缺少参数：dialogueId");
        }
        if (!jsonObject.containsKey("content")) {
            return fail("", "缺少参数：content");
        }
        if (!jsonObject.containsKey("image")) {
            return fail("", "缺少参数：image");
        }
        String phone = jsonObject.getString("phone");
        UserEntity user = this.userService.findUserByPhone(phone);
        if (user == null){
            return fail(null,"用户不存在");
        }
        String content = jsonObject.getString("content");
        String image = jsonObject.getString("image");
        String dialogueId = jsonObject.getString("dialogueId");
        if (dialogueId.equals("") || dialogueId == null){
            if (content.length() > 50){
                content = content.substring(0,50);
            }
            //新建对话
            DialogueEntity dialogueEntity = new DialogueEntity();
            dialogueEntity.setDialogueName(content);
            dialogueEntity.setPhone(phone);
            dialogueEntity.setStatus(1);
            dialogueEntity.setCreateTime(DateUtil.format(new Date(), "yyyy-MM-dd HH:mm:ss.SSS"));
            dialogueEntity.setUpdateTime(DateUtil.format(new Date(), "yyyy-MM-dd HH:mm:ss.SSS"));
            dialogueEntity = this.dialogueService.save(dialogueEntity);
            dialogueId = dialogueEntity.getId();
        }
        ChatHistoryEntity chatHistoryEntity = new ChatHistoryEntity();
        chatHistoryEntity.setDialogueId(dialogueId);
        chatHistoryEntity.setRole("user");
        chatHistoryEntity.setContent(content);
        chatHistoryEntity.setImage(image);
        chatHistoryEntity.setStatus(1);
        chatHistoryEntity.setCreateTime(DateUtil.format(new Date(), "yyyy-MM-dd HH:mm:ss.SSS"));
        chatHistoryEntity.setUpdateTime(DateUtil.format(new Date(), "yyyy-MM-dd HH:mm:ss.SSS"));
        this.chatHistoryService.save(chatHistoryEntity);
        //调用模型相关操作
        List<ChatHistoryEntity> chatHistoryEntities = this.chatHistoryService.findChatHistoryByDialogueIdAndStatus(dialogueId);
        JSONArray messages = new JSONArray();
        for (ChatHistoryEntity chatHistory : chatHistoryEntities) {
            JSONArray contents = new JSONArray();
            JSONObject jsonText = new JSONObject();
            jsonText.put("type","text");
            jsonText.put("text",chatHistory.getContent());
            if (chatHistory.getImage() != null && !chatHistory.getImage().equals("")) {
                JSONObject jsonImage = new JSONObject();
                jsonImage.put("type","image_url");
                JSONObject jsonUrl = new JSONObject();
                jsonUrl.put("url",chatHistory.getImage());
                jsonImage.put("image_url",jsonUrl);
                contents.add(jsonImage);
            }
            contents.add(jsonText);
            JSONObject jsonRole = new JSONObject();
            jsonRole.put("role",chatHistory.getRole());
            jsonRole.put("content",contents);
            messages.add(jsonRole);
        }
        JSONObject jsonChat = new JSONObject();
        jsonChat.put("messages",messages);

        String url = Constant.INVOKE_IP_PROT + Constant.CHAT_PATH;
        String data = HttpClientChat(jsonChat,url);
        JSONObject jsonSystem = JSONObject.parseObject(data);
        if (jsonSystem == null || !jsonSystem.containsKey("response")) {
            return fail("", "模型服务内部错误");
        }
        content = jsonSystem.getString("response");
        chatHistoryEntity = new ChatHistoryEntity();
        chatHistoryEntity.setDialogueId(dialogueId);
        chatHistoryEntity.setRole("system");
        chatHistoryEntity.setContent(content);
        chatHistoryEntity.setStatus(1);
        chatHistoryEntity.setCreateTime(DateUtil.format(new Date(), "yyyy-MM-dd HH:mm:ss.SSS"));
        chatHistoryEntity.setUpdateTime(DateUtil.format(new Date(), "yyyy-MM-dd HH:mm:ss.SSS"));
        chatHistoryEntity = this.chatHistoryService.save(chatHistoryEntity);

        return success(chatHistoryEntity);
    }

    @GetMapping("/findDialogues")
    @ApiOperation(value = "查询对话列表",notes = "参数包括：pageNum:页码, pageSize:数量, search:搜索内容")
    public Result findDialogues(@RequestParam int pageNum, @RequestParam int pageSize, @RequestParam String search, HttpServletRequest request){
        String token = request.getHeader("Authorization");
        String[] tokens = token.split(" ");
        if (tokens.length != 2) {
            token = tokens[0];
        }else {
            token = tokens[1];
        }
        JWTUtil.verify(token);
        DecodedJWT verify = JWTUtil.verify(token);
        String phone = verify.getClaim("phone").asString();
        Page<DialogueEntity> dialogueEntities = dialogueService.findByPhoneAndStatusAndSearch(pageNum,pageSize,phone,1,search);
        return success(dialogueEntities);
    }

    @PostMapping("/updateDialogues")
    @ApiOperation(value = "修改对话信息（重命名和删除）",notes = "Dialogues对象,修改对话名称时只需将修改后的名称放入dialogueName中；删除对话名称时只需将status的值改为0")
    public Result updateDialogues(@RequestBody String json) {
        DialogueEntity dialogueEntity = JSONObject.parseObject(json,DialogueEntity.class);
        dialogueEntity.setUpdateTime(DateUtil.format(new Date(), "yyyy-MM-dd HH:mm:ss.SSS"));
        this.dialogueService.save(dialogueEntity);
        return success("修改成功");
    }

    public static String HttpClientChat(JSONObject jsonChat, String url) throws UnsupportedEncodingException {
        System.out.println("11:"+jsonChat);
        CloseableHttpClient httpclient = HttpClients.createDefault();
        HttpPost httpPost = new HttpPost(url);
        StringEntity entity = new StringEntity(jsonChat.toString(),"UTF-8");
        entity.setContentEncoding("UTF-8");
        entity.setContentType("application/json");
        httpPost.setEntity(entity);
        HttpResponse response = null;
        String responseData = null;
        try {
            response = httpclient.execute(httpPost);
            if (response.getStatusLine().getStatusCode() == 200){
                responseData =  EntityUtils.toString(response.getEntity());
            }
        } catch (IOException e) {
            e.printStackTrace();
        } finally {
            try {
                httpclient.close();
            } catch (IOException e) {
                e.printStackTrace();
            }
        }

        return responseData;
    }

}
